# 2. Lists of generic items encapsulated with a selection status

Date: 2021-05-05

## Status

Superceded by [4. revert exposing SelectableItem publicly](0004-revert-exposing-selectableitem-publicly.md)

Handling of selection specific properties is described in [3. Properties specific for selectable lists](0003-properties-specific-for-selectable-lists.md)

## Context

Applications with a GUI (e.g. Swing- or JavaFX- based) often use lists with single or multiple items being selectable.
Selection can be changed by a mouse click on an item, and multiple selections can be extended holding Ctrl or Shift key with a mouse click.
GUI frameworks contain ready to use components for those standard use cases.

Downside is that such selections are very volatile (a single, possibly erroneous click without a modifier key destroys all previous selection, making it difficult to create a complex selection)
and sometimes both a single and a multiple selection of items is needed
(e.g. in a large list containing images, represented by thumbnails, a single image should be selected for full size display and at the same time a subset of images should be selectable for file copy operations).

Obvious solution in the view is to pair each list item (the thumbnail in the previous example) with a checkbox to handle the selection status.

But what about the data model? JavaFX provides the `ObservableList<E>` interface to handle the list data internally including event handling to listen for changes in the list data.

When dealing with item selection, we need to encapsulate the generic item with a boolean selection flag and also provide an additional event handling for selection changes.
We also need some means for efficiently providing the number of selected elements at any time and providing the information, whether all or none of the elements in the list are selected.

Note, that `ObservableList<E>` provides a `size()` method to query the number of list elements, but it does *NOT* provide the `ReadOnlyIntegerProperty sizeProperty()` which is located in the `ListExpression<E>` interface (e.g. implemented by the `ListProperty<E>` class).

The questions, which additional properties to implement concerning list item selection, and how and where, are dealt with in a separate ADR:

[3. Properties specific for selectable lists](0003-properties-specific-for-selectable-lists.md).

## Decision

We will implement a set of related types around a central interface `SelectableList<E>` in the package `de.bernd_michaely.common.desktop.fx.collections`.

The interface extends the JavaFX `ObservableList<SelectableItem<E>>` type, where `SelectableItem<E>` is an interface describing the encapsulation of a generic item with a selection status.

The integration with `ObservableList<E>` will allow the use of `TransformationList<E>`. That is the selection must take into consideration list transformations like sortings and filters.

The implementation of the `SelectableList<E>` interface will be provided exclusively through a factory class `CommonFXCollections`.

The implementation of the `SelectableItem<E>` interface will be encapsulated, e.g. by a private inner class. This is important to have full control over handling of selection changes. (When available, this interface is a candidate for a sealed type.)

## Consequences

We will have available a ready to use software component providing a list data model for generic items encapsulated with a selection status including event handling mechanism and suitable properties.

The software component will provide the usual advantages of modularization, including reusability, avoiding the reinvention of the wheel and central maintenance, leading to better testing and documentation, leading to better reusability and maintenance.

