# 3. Properties specific for selectable lists

Date: 2021-05-05

## Status

Accepted

Describes handling of selection specific properties for [2. Lists of generic items encapsulated with a selection status](0002-lists-of-generic-items-encapsulated-with-a-selection-status.md)

## Context

The lists introduced in the above ADR need additional means to provide properties to indicate the number of selected items or the information, whether all items are selected.

While a simple implementation (react to changes and always count) would result in `O(n)` time, we aim to achieve an implementation with `O(1)` time.

Note, that those properties also must take into consideration the use of `TransformationList<E>`, that is, e.g. count only the selected items currently visible through a filter, and then provide the count of all selected items in the list again, when the filter is reset.

## Decision

To provide selection related properties for the `SelectableList<E>` introduced in ADR 2 (see above), we define and implement a `ListSelectionHandler<E>` interface.

Instances are provided exclusively through a factory class `CommonFXCollections`.

We will provide a mechanism for handling selection changes and counting by the internal `SelectionControl<E>` class. This control object holds a weak reference to the list, which it also passes to the event handling, to avoid memory leaks.

## Consequences

We will have available a ready to use software component providing a list data model for generic items encapsulated with a selection status including event handling mechanism and suitable properties.

