/* Created on Dec 11, 2020 */
package de.bernd_michaely.common.desktop.fx.collections.selection;

import java.util.Collection;
import javafx.collections.transformation.TransformationList;

/**
 * Class with factory methods for creation of instances of SelectableLists and
 * related types.
 *
 * @author Bernd Michaely (info@bernd-michaely.de)
 */
public class SelectableListFactory
{
	/**
	 * Private constructor, this class contains static factory methods only.
	 */
	private SelectableListFactory()
	{
	}

	/**
	 * Returns a SelectableList implementation.
	 *
	 * @param <E> the list element type
	 * @return a SelectableList instance
	 */
	public static <E> SelectableList<E> selectableList()
	{
		return new SelectableListImpl<>();
	}

	/**
	 * Returns a SelectableList implementation.
	 *
	 * @param <E> the list element type
	 * @param src a given collection the list is initially filled with
	 * @return a SelectableList instance
	 * @throws NullPointerException if src is null
	 */
	public static <E> SelectableList<E> selectableList(Collection<? extends E> src)
	{
		return new SelectableListImpl<>(src);
	}

	/**
	 * Returns a SelectableListSelectionHandler implementation.
	 *
	 * @param <E>        the list element type
	 * @param sourceList a given list the returned instance is based on
	 * @return a SelectableListSelectionHandler instance
	 * @throws NullPointerException if sourceList is null
	 */
	public static <E> ListSelectionHandler<E> listSelectionHandler(SelectableList<E> sourceList)
	{
		return new ListSelectionHandlerImpl<>(sourceList);
	}

	/**
	 * Returns a SelectableListSelectionHandler implementation.
	 *
	 * @param <E>                the list element type
	 * @param sourceList         a given list the returned instance is based on
	 * @param transformationList a transformation list (or a chain thereof) to be
	 *                           applied to the source list
	 * @return a ListSelectionHandler instance
	 * @throws NullPointerException if sourceList is null
	 */
	public static <E> ListSelectionHandler<E> listSelectionHandler(SelectableList<E> sourceList,
		TransformationList<E, E> transformationList)
	{
		return new ListSelectionHandlerImpl<>(sourceList, transformationList);
	}
}
